@@ -15,15 +15,17 @@
 module Data.Aeson.Parser.Internal
     (
     -- * Lazy parsers
-      json
+      json, jsonEOF
     , value
     , jstring
     -- * Strict parsers
-    , json'
+    , json', jsonEOF'
     , value'
     -- * Helpers
     , decodeWith
+    , decodeStrictWith
     , eitherDecodeWith
+    , eitherDecodeStrictWith
     ) where
 
 import Blaze.ByteString.Builder (fromByteString, toByteString)
@@ -37,7 +39,7 @@ import Data.ByteString as B
 import Data.Char (chr)
 import Data.Monoid (mappend, mempty)
 import Data.Text as T
-import Data.Text.Encoding (decodeUtf8)
+import Data.Text.Encoding (decodeUtf8')
 import Data.Vector as Vector hiding ((++))
 import Data.Word (Word8)
 import qualified Data.Attoparsec as A
@@ -175,11 +177,16 @@ jstring_ = {-# SCC "jstring_" #-} do
                                         then Nothing
                                         else Just (c == backslash)
   _ <- A.word8 doubleQuote
-  if backslash `B.elem` s
-    then case Z.parse unescape s of
-           Right r  -> return (decodeUtf8 r)
-           Left err -> fail err
-    else return (decodeUtf8 s)
+  s' <- if backslash `B.elem` s
+        then case Z.parse unescape s of
+            Right r  -> return r
+            Left err -> fail err
+         else return s
+
+  case decodeUtf8' s' of
+      Right r  -> return r
+      Left err -> fail $ show err
+
 {-# INLINE jstring_ #-}
 
 unescape :: Z.Parser ByteString
@@ -239,7 +246,18 @@ decodeWith p to s =
       _          -> Nothing
 {-# INLINE decodeWith #-}
 
-eitherDecodeWith :: Parser Value -> (Value -> Result a) -> L.ByteString -> Either String a
+decodeStrictWith :: Parser Value -> (Value -> Result a) -> B.ByteString
+                 -> Maybe a
+decodeStrictWith p to s =
+    case A.parse p s of
+      A.Done _ v -> case to v of
+                      Success a -> Just a
+                      _         -> Nothing
+      _          -> Nothing
+{-# INLINE decodeStrictWith #-}
+
+eitherDecodeWith :: Parser Value -> (Value -> Result a) -> L.ByteString
+                 -> Either String a
 eitherDecodeWith p to s =
     case L.parse p s of
       L.Done _ v -> case to v of
@@ -248,6 +266,17 @@ eitherDecodeWith p to s =
       L.Fail _ _ msg -> Left msg
 {-# INLINE eitherDecodeWith #-}
 
+eitherDecodeStrictWith :: Parser Value -> (Value -> Result a) -> B.ByteString
+                       -> Either String a
+eitherDecodeStrictWith p to s =
+    case A.parse p s of
+      A.Done _ v -> case to v of
+                      Success a -> Right a
+                      Error msg -> Left msg
+      A.Fail _ _ msg -> Left msg
+      A.Partial _    -> Left "incomplete input"
+{-# INLINE eitherDecodeStrictWith #-}
+
 -- $lazy
 --
 -- The 'json' and 'value' parsers decouple identification from
@@ -266,3 +295,13 @@ eitherDecodeWith p to s =
 -- The 'json'' and 'value'' parsers combine identification with
 -- conversion.  They consume more CPU cycles up front, but have a
 -- smaller memory footprint.
+
+-- | Parse a top-level JSON value followed by optional whitespace and
+-- end-of-input.  See also: 'json'.
+jsonEOF :: Parser Value
+jsonEOF = json <* skipSpace <* endOfInput
+
+-- | Parse a top-level JSON value followed by optional whitespace and
+-- end-of-input.  See also: 'json''.
+jsonEOF' :: Parser Value
+jsonEOF' = json' <* skipSpace <* endOfInput
