@@ -32,6 +32,15 @@ module Data.Aeson.Types
     , parseEither
     , parseMaybe
     , ToJSON(..)
+    , modifyFailure
+
+#ifdef GENERICS
+    -- ** Generic JSON classes
+    , GFromJSON(..)
+    , GToJSON(..)
+    , genericToJSON
+    , genericParseJSON
+#endif
 
     -- * Inspecting @'Value's@
     , withObject
@@ -46,6 +55,12 @@ module Data.Aeson.Types
     , (.:?)
     , (.!=)
     , object
+
+    -- * Generic and TH encoding configuration
+    , Options(..)
+    , SumEncoding(..)
+    , defaultOptions
+    , defaultTaggedObject
     ) where
 
 import Data.Aeson.Types.Class
